@@ -1,5 +1,6 @@
 const writeToStream = require('./writeToStream')
 const EventEmitter = require('events')
+const { Buffer } = require('buffer')
 
 function generate (packet, opts) {
   const stream = new Accumulator()
@@ -47,6 +48,10 @@ class Accumulator extends EventEmitter {
 
     return result
   }
+
+  destroy (err) {
+    if (err) this.emit('error', err)
+  }
 }
 
 module.exports = generate
